package be.authtest.vaadin.ui;

import java.io.IOException;
import java.security.Principal;
import java.util.Iterator;
import java.util.Set;

import javax.inject.Inject;
import javax.security.auth.Subject;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.scribe.builder.ServiceBuilder;
import org.scribe.oauth.OAuthService;

import com.vaadin.annotations.Theme;
import com.vaadin.annotations.Widgetset;
import com.vaadin.cdi.CDIUI;
import com.vaadin.cdi.access.JaasAccessControl;
import com.vaadin.server.ExternalResource;
import com.vaadin.server.RequestHandler;
import com.vaadin.server.VaadinRequest;
import com.vaadin.server.VaadinResponse;
import com.vaadin.server.VaadinServletResponse;
import com.vaadin.server.VaadinSession;
import com.vaadin.ui.Link;
import com.vaadin.ui.Panel;
import com.vaadin.ui.UI;
import com.vaadin.v7.ui.VerticalLayout;

import be.authtest.vaadin.ejb.AuthService;
import be.authtest.vaadin.oauth.Google2Api;
import be.authtest.vaadin.oauth.GoogleCallbackHandler;

/**
 * @author LDK
 *
 */
@Theme("valo")
@CDIUI("login2")
@SuppressWarnings("serial")
@Widgetset("com.vaadin.v7.Vaadin7WidgetSet")
public class GoogleLoginUI extends UI implements RequestHandler {
	private static final Logger logger = LogManager.getLogger(GoogleLoginUI.class.getName());
	OAuthService service;
	String redirectUrl = "http://localhost:8080/vaadin/secure/admin/dashboard";
	private Link gplusLoginButton;

	@Inject
	AuthService playerService;
	
	@Override
	protected void init(VaadinRequest request) {
		final Panel loginPanel = new Panel("login");
		final VerticalLayout layout = new VerticalLayout();
		layout.setSizeUndefined();
		layout.setMargin(true);
		loginPanel.setContent(layout);
		loginPanel.setSizeUndefined();

		/*
		 * TextField username = new TextField();
		 * username.setInputPrompt("Username"); PasswordField password = new
		 * PasswordField(); password.setInputPrompt("Password");
		 */

		//logger.debug("Authentication info {} ",this.playerService.getSecurityInfo());
		
		logger.debug("Remote user  = {}", request.getRemoteUser());
		
		logger.debug("Principal  = {}", request.getUserPrincipal());
		
		
		
		this.service = createService();
		String url = this.service.getAuthorizationUrl(null);
		this.gplusLoginButton = new Link("Login with Google", new ExternalResource(url));
		setContent(new VerticalLayout(gplusLoginButton));

		VaadinSession.getCurrent().addRequestHandler(this);
		
		try {
			Principal principal = JaasAccessControl.getCurrentRequest().getUserPrincipal();
			if (principal != null) {
				JaasAccessControl.logout();
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}


	@Override
	public boolean handleRequest(VaadinSession session, VaadinRequest request, VaadinResponse response) throws IOException {
		try {
			if (request.getParameter("code") != null) {
			
				//logger.debug("Authentication info {} ",this.playerService.getSecurityInfo());
				
				Principal principal = JaasAccessControl.getCurrentRequest().getUserPrincipal();
				if (principal != null) {
					JaasAccessControl.logout();
				}
				
				String code = request.getParameter("code");
				GoogleCallbackHandler cbh = new GoogleCallbackHandler(code);
				LoginContext loginContext = new LoginContext("GoogleLogin", new Subject(), cbh);
				loginContext.login();
				Subject subject = loginContext.getSubject();
				
				for (Principal p : subject.getPrincipals()) {
					System.out.println("Principal = " +  p);	
				}

				principal = JaasAccessControl.getCurrentRequest().getUserPrincipal();
				
				VaadinSession.getCurrent().removeRequestHandler(this);
				((VaadinServletResponse) response).getHttpServletResponse().sendRedirect(this.redirectUrl);
			}
		}
		catch (LoginException e) {
			logger.error("Unable to log in {}", e.getMessage());
		}
		catch (Exception e2) {
			e2.printStackTrace();
		}
			
		return true;
		
	}

    private OAuthService createService() {
        ServiceBuilder sb = new ServiceBuilder();
        sb.provider(Google2Api.class);
        sb.apiKey(this.gpluskey);
        sb.apiSecret(this.gplussecret);
        sb.scope("email");
        sb.callback("http://localhost:8080/vaadin/login/login2");
        return sb.build();
    }

	private String gpluskey = "**********************";
	private String gplussecret = "*************************";

}
